<?php
/**
 * Created by PhpStorm.
 * User: leeyifiei
 * Date: 15-10-27
 * Time: 上午12:42
 */

class Tool
{

    //保存类实例的静态成员变量
    private static $_instance;

    //创建__clone方法防止对象被复制克隆
    public function __clone()
    {
        throw new ErrorException( 'Clone is not allow', E_USER_ERROR );
    }

    public static function getInstance()
    {
        if ( !(self::$_instance instanceof self) ) {
            self::$_instance = new self;
        }

        return self::$_instance;
    }

    /**
     * lang
     * @param $message
     * @param array $vars
     * @return string
     */
    public function lang( $message, $vars = array() )
    {
        $template = 'message';
        if( strtoupper( Yii::app()->charset ) == 'GBK' )
        {
            $template = 'message_gbk';
        }

        if( !empty($vars) )
        {
            return Yii::t( $template, $message, $vars );
        }
        else
        {
            return Yii::t( $template, $message );
        }
    }

    /**
     * 模板语言包
     * @param $message
     * @param array $vars
     * @param string $file
     * @return string
     */
    public function tplLang( $message, $vars = array(), $file = 'tpl' )
    {
        $template = $file;

        if( strtoupper( Yii::app()->charset ) == 'GBK' )
        {
            $template = $file . '_gbk';
        }

        if( !empty($vars) )
        {
            return Yii::t( $template, $message, $vars );
        }
        else
        {
            return Yii::t( $template, $message );
        }
    }

    /**
     * 输出JSON数据
     * @param $data
     * @return string
     */
    public function success_json( $data )
    {
        $return = array(
            'errCode' => 0,
            'errMessage' => '',
            'data' => $data
        );

        echo CJSON::encode( $return );
        Yii::app()->end(0);
    }

    /**
     * 输出JSON数据
     * @param $data
     * @return string
     */
    public function fail_json( $errCode, $errMessage )
    {
        echo CJSON::encode(array(
            'errCode' => $errCode,
            'errMessage' => $errMessage
        ));
        Yii::app()->end();
    }

    /**
     * 错误提示
     * @param $status
     * @param $args
     */
    public function fail_for_discuz_model()
    {
        $args = func_get_args();

        $errorMessage = '';
        if( is_array($args) )
        {
            if( count($args) > 1 )
            {
                $errorMessage = lang('message', $args[0], $args[2] );
            }
            else
            {
                $errorMessage = lang( 'message' , $args[0] );
            }
        }
        else
        {
            $errorMessage = lang( 'message', $args );
        }

        echo CJSON::encode(array(
            'errCode' => Code::DISCUZ_MODEL_ERROR,
            'errMessage' => $errorMessage
        ));

        Yii::app()->end();
    }

    public function convert2charset( $data, $in = 'GBK', $out = 'UTF-8' )
    {
        if( is_array($data) )
        {
            return array_map( array('Tool', 'convert2charset'), $data );
        }
        else
        {
            if( is_string( $data) )
            {
                return diconv($data, $in, $out);
//                return iconv( $in, $out, $data );
            }
            else
            {
                return $data;
            }
        }


    }

    /**
     * 清除HTML标签
     * @param $str
     * @param string $charset
     * @param bool|false $transBr
     * @return mixed
     */
    public function stripHtmlTags( $str, $transBr=false )
    {
        if ($transBr) {
            $str = str_replace('<br>', "\r\n", $str);
            $str = str_replace('<br />', "\r\n", $str);
        }
        $str = preg_replace('/<.*?>/', '', $str);
        $str = str_replace('&nbsp;', ' ', $str);
        return $str;
    }

    public function authcode($string, $operation, $key = '', $expiry = 0)
    {
        $ckey_length = 4;
        $key = md5($key);

        $keya = md5(substr($key, 0, 16));
        $keyb = md5(substr($key, 16, 16));
        $keyc = $ckey_length ? ($operation == 'DECODE' ? substr($string, 0, $ckey_length): substr(md5(microtime()), -$ckey_length)) : '';

        $cryptkey = $keya.md5($keya.$keyc);
        $key_length = strlen($cryptkey);

        $string = $operation == 'DECODE' ? base64_decode(substr($string, $ckey_length)) : sprintf('%010d', $expiry ? $expiry + time() : 0).substr(md5($string.$keyb), 0, 16).$string;
        $string_length = strlen($string);

        $result = '';
        $box = range(0, 255);

        $rndkey = array();
        for($i = 0; $i <= 255; $i++) {
            $rndkey[$i] = ord($cryptkey[$i % $key_length]);
        }

        for($j = $i = 0; $i < 256; $i++) {
            $j = ($j + $box[$i] + $rndkey[$i]) % 256;
            $tmp = $box[$i];
            $box[$i] = $box[$j];
            $box[$j] = $tmp;
        }

        for($a = $j = $i = 0; $i < $string_length; $i++) {
            $a = ($a + 1) % 256;
            $j = ($j + $box[$a]) % 256;
            $tmp = $box[$a];
            $box[$a] = $box[$j];
            $box[$j] = $tmp;
            $result .= chr(ord($string[$i]) ^ ($box[($box[$a] + $box[$j]) % 256]));
        }

        if($operation == 'DECODE') {
            if((substr($result, 0, 10) == 0 || substr($result, 0, 10) - time() > 0) && substr($result, 10, 16) == substr(md5(substr($result, 26).$keyb), 0, 16)) {
                return substr($result, 26);
            } else {
                return '';
            }
        } else {
            return $keyc.str_replace('=', '', base64_encode($result));
        }
    }

    /**
     * 时间戳转化，时间友好显示
     *
     * @param $sTime
     * @param int $format
     * @return bool|string
     */
    public function friendly($sTime,$format=0) {
        //sTime=源时间，cTime=当前时间，dTime=时间差
        $cTime        =    time();
        $dTime        =    $cTime - $sTime;
        $dDay        =    intval(date("Ymd",$cTime)) - intval(date("Ymd",$sTime));
        //normal：n秒前，n分钟前，n小时前，日期
        if($dTime == 0){
            return "1秒前";
        }elseif( $dTime < 60 && $dTime > 0 ){
            return $dTime."秒前";
        }
        elseif( $dTime < 3600 && $dTime > 0 ){
            return intval($dTime/60)."分钟前";
        }
        elseif( $dTime >= 3600 && $dDay == 0 )
        {
            return intval($dTime/3600)."小时前";
        }
        elseif( $dDay == 1 )
        {
            return date("昨天 H:i",$sTime);
        }
        elseif( $dDay == 2 )
        {
            return date("前天 H:i",$sTime);
        }
        elseif($format == 1){
            return date("m-d H:i",$sTime);
        }else{
            return date("m-d",$sTime);
        }
        //full: Y-m-d , H:i:s
    }

    /**
     * 验证手机格式
     * @param $phone
     * @param $type
     * @return bool
     */
    public function VerifyMobileNumber($phone, $type)
    {
        if ($type == 1 && is_int(strpos($phone, ' '))) {
            // 带有国际区号
            if (preg_match('/(\+\d{1,7})\ (\d{3,20}.*?)/', $phone)) {
                return true;
            }
        } else {
            if(preg_match('/^1[3456789]{1}\d{9}$/', $phone)) {
                return true;
            }
        }
        return false;
    }

    /*
     * array_column 支持
     */
    public static function Array_Column($array,$column_key,$index_key = null,$is_array = null){
        $return = array();
        if(!is_array($array) || empty($array))
            return $return;
        foreach($array as $val){
            if(isset($val[$column_key])){
                if($index_key && isset($val[$index_key])){
                    if($is_array){
                        $return[$val[$index_key]][] = $val[$column_key];
                    }else{
                        $return[$val[$index_key]] = $val[$column_key];
                    }
                }else{
                    $return[] = $val[$column_key];
                }
            }
        }
        return $return;
    }

}